@@ -1,6 +1,6 @@
+from collections.abc import Generator
+from collections.abc import Sequence
 from typing import Callable
-from typing import List
-from typing import Sequence
 from typing import TypeVar
 
 import pytest
@@ -9,15 +9,16 @@ from pluggy import HookimplMarker
 from pluggy import HookspecMarker
 from pluggy import PluginManager
 from pluggy import PluginValidationError
-from pluggy._hooks import _HookCaller
+from pluggy._hooks import HookCaller
 from pluggy._hooks import HookImpl
 
+
 hookspec = HookspecMarker("example")
 hookimpl = HookimplMarker("example")
 
 
 @pytest.fixture
-def hc(pm: PluginManager) -> _HookCaller:
+def hc(pm: PluginManager) -> HookCaller:
     class Hooks:
         @hookspec
         def he_method1(self, arg: object) -> None:
@@ -31,7 +32,7 @@ FuncT = TypeVar("FuncT", bound=Callable[..., object])
 
 
 class AddMeth:
-    def __init__(self, hc: _HookCaller) -> None:
+    def __init__(self, hc: HookCaller) -> None:
         self.hc = hc
 
     def __call__(
@@ -57,15 +58,15 @@ class AddMeth:
 
 
 @pytest.fixture
-def addmeth(hc: _HookCaller) -> AddMeth:
+def addmeth(hc: HookCaller) -> AddMeth:
     return AddMeth(hc)
 
 
-def funcs(hookmethods: Sequence[HookImpl]) -> List[Callable[..., object]]:
+def funcs(hookmethods: Sequence[HookImpl]) -> list[Callable[..., object]]:
     return [hookmethod.function for hookmethod in hookmethods]
 
 
-def test_adding_nonwrappers(hc: _HookCaller, addmeth: AddMeth) -> None:
+def test_adding_nonwrappers(hc: HookCaller, addmeth: AddMeth) -> None:
     @addmeth()
     def he_method1() -> None:
         pass
@@ -81,7 +82,7 @@ def test_adding_nonwrappers(hc: _HookCaller, addmeth: AddMeth) -> None:
     assert funcs(hc.get_hookimpls()) == [he_method1, he_method2, he_method3]
 
 
-def test_adding_nonwrappers_trylast(hc: _HookCaller, addmeth: AddMeth) -> None:
+def test_adding_nonwrappers_trylast(hc: HookCaller, addmeth: AddMeth) -> None:
     @addmeth()
     def he_method1_middle() -> None:
         pass
@@ -97,7 +98,7 @@ def test_adding_nonwrappers_trylast(hc: _HookCaller, addmeth: AddMeth) -> None:
     assert funcs(hc.get_hookimpls()) == [he_method1, he_method1_middle, he_method1_b]
 
 
-def test_adding_nonwrappers_trylast3(hc: _HookCaller, addmeth: AddMeth) -> None:
+def test_adding_nonwrappers_trylast3(hc: HookCaller, addmeth: AddMeth) -> None:
     @addmeth()
     def he_method1_a() -> None:
         pass
@@ -122,7 +123,7 @@ def test_adding_nonwrappers_trylast3(hc: _HookCaller, addmeth: AddMeth) -> None:
     ]
 
 
-def test_adding_nonwrappers_trylast2(hc: _HookCaller, addmeth: AddMeth) -> None:
+def test_adding_nonwrappers_trylast2(hc: HookCaller, addmeth: AddMeth) -> None:
     @addmeth()
     def he_method1_middle() -> None:
         pass
@@ -138,7 +139,7 @@ def test_adding_nonwrappers_trylast2(hc: _HookCaller, addmeth: AddMeth) -> None:
     assert funcs(hc.get_hookimpls()) == [he_method1, he_method1_middle, he_method1_b]
 
 
-def test_adding_nonwrappers_tryfirst(hc: _HookCaller, addmeth: AddMeth) -> None:
+def test_adding_nonwrappers_tryfirst(hc: HookCaller, addmeth: AddMeth) -> None:
     @addmeth(tryfirst=True)
     def he_method1() -> None:
         pass
@@ -154,26 +155,26 @@ def test_adding_nonwrappers_tryfirst(hc: _HookCaller, addmeth: AddMeth) -> None:
     assert funcs(hc.get_hookimpls()) == [he_method1_middle, he_method1_b, he_method1]
 
 
-def test_adding_wrappers_ordering(hc: _HookCaller, addmeth: AddMeth) -> None:
+def test_adding_wrappers_ordering(hc: HookCaller, addmeth: AddMeth) -> None:
     @addmeth(hookwrapper=True)
     def he_method1():
-        yield
+        yield  # pragma: no cover
 
     @addmeth(wrapper=True)
     def he_method1_fun():
-        yield
+        yield  # pragma: no cover
 
     @addmeth()
     def he_method1_middle():
-        return
+        return  # pragma: no cover
 
     @addmeth(hookwrapper=True)
     def he_method3_fun():
-        yield
+        yield  # pragma: no cover
 
     @addmeth(hookwrapper=True)
     def he_method3():
-        yield
+        yield  # pragma: no cover
 
     assert funcs(hc.get_hookimpls()) == [
         he_method1_middle,
@@ -184,94 +185,86 @@ def test_adding_wrappers_ordering(hc: _HookCaller, addmeth: AddMeth) -> None:
     ]
 
 
-def test_adding_wrappers_ordering_tryfirst(hc: _HookCaller, addmeth: AddMeth) -> None:
+def test_adding_wrappers_ordering_tryfirst(hc: HookCaller, addmeth: AddMeth) -> None:
     @addmeth(hookwrapper=True, tryfirst=True)
     def he_method1():
-        yield
+        yield  # pragma: no cover
 
     @addmeth(hookwrapper=True)
     def he_method2():
-        yield
+        yield  # pragma: no cover
 
     @addmeth(wrapper=True, tryfirst=True)
     def he_method3():
-        yield
+        yield  # pragma: no cover
 
     assert funcs(hc.get_hookimpls()) == [he_method2, he_method1, he_method3]
 
 
-def test_adding_wrappers_complex(hc: _HookCaller, addmeth: AddMeth) -> None:
+def test_adding_wrappers_complex(hc: HookCaller, addmeth: AddMeth) -> None:
     assert funcs(hc.get_hookimpls()) == []
 
     @addmeth(hookwrapper=True, trylast=True)
     def m1():
-        yield
+        yield  # pragma: no cover
 
     assert funcs(hc.get_hookimpls()) == [m1]
 
     @addmeth()
-    def m2() -> None:
-        ...
+    def m2() -> None: ...
 
     assert funcs(hc.get_hookimpls()) == [m2, m1]
 
     @addmeth(trylast=True)
-    def m3() -> None:
-        ...
+    def m3() -> None: ...
 
     assert funcs(hc.get_hookimpls()) == [m3, m2, m1]
 
     @addmeth(hookwrapper=True)
-    def m4() -> None:
-        ...
+    def m4() -> None: ...
 
     assert funcs(hc.get_hookimpls()) == [m3, m2, m1, m4]
 
     @addmeth(wrapper=True, tryfirst=True)
     def m5():
-        yield
+        yield  # pragma: no cover
 
     assert funcs(hc.get_hookimpls()) == [m3, m2, m1, m4, m5]
 
     @addmeth(tryfirst=True)
-    def m6() -> None:
-        ...
+    def m6() -> None: ...
 
     assert funcs(hc.get_hookimpls()) == [m3, m2, m6, m1, m4, m5]
 
     @addmeth()
-    def m7() -> None:
-        ...
+    def m7() -> None: ...
 
     assert funcs(hc.get_hookimpls()) == [m3, m2, m7, m6, m1, m4, m5]
 
     @addmeth(wrapper=True)
     def m8():
-        yield
+        yield  # pragma: no cover
 
     assert funcs(hc.get_hookimpls()) == [m3, m2, m7, m6, m1, m4, m8, m5]
 
     @addmeth(trylast=True)
-    def m9() -> None:
-        ...
+    def m9() -> None: ...
 
     assert funcs(hc.get_hookimpls()) == [m9, m3, m2, m7, m6, m1, m4, m8, m5]
 
     @addmeth(tryfirst=True)
-    def m10() -> None:
-        ...
+    def m10() -> None: ...
 
     assert funcs(hc.get_hookimpls()) == [m9, m3, m2, m7, m6, m10, m1, m4, m8, m5]
 
     @addmeth(hookwrapper=True, trylast=True)
-    def m11() -> None:
-        ...
+    def m11() -> None: ...
 
     assert funcs(hc.get_hookimpls()) == [m9, m3, m2, m7, m6, m10, m11, m1, m4, m8, m5]
 
     @addmeth(wrapper=True)
     def m12():
-        yield
+        yield  # pragma: no cover
 
     assert funcs(hc.get_hookimpls()) == [
         m9,
@@ -289,8 +282,7 @@ def test_adding_wrappers_complex(hc: _HookCaller, addmeth: AddMeth) -> None:
     ]
 
     @addmeth()
-    def m13() -> None:
-        ...
+    def m13() -> None: ...
 
     assert funcs(hc.get_hookimpls()) == [
         m9,
@@ -413,7 +405,7 @@ def test_hookrelay_registration_by_specname_raises(pm: PluginManager) -> None:
     class Plugin:
         @hookimpl(specname="hello")
         def foo(self, arg: int, too, many, args) -> int:
-            return arg + 1
+            return arg + 1  # pragma: no cover
 
     with pytest.raises(PluginValidationError):
         pm.register(Plugin())
@@ -423,7 +415,7 @@ def test_hookrelay_registration_by_specname_raises(pm: PluginManager) -> None:
     class Plugin2:
         @hookimpl(specname="bar")
         def hello(self, arg: int) -> int:
-            return arg + 1
+            return arg + 1  # pragma: no cover
 
     pm.register(Plugin2())
     with pytest.raises(PluginValidationError):
@@ -448,3 +440,74 @@ def test_hook_conflict(pm: PluginManager) -> None:
         "Hook 'conflict' is already registered within namespace "
         "<class 'test_hookcaller.test_hook_conflict.<locals>.Api1'>"
     )
+
+
+def test_call_extra_hook_order(hc: HookCaller, addmeth: AddMeth) -> None:
+    """Ensure that call_extra is calling hooks in the right order."""
+    order = []
+
+    @addmeth(tryfirst=True)
+    def method1() -> str:
+        order.append("1")
+        return "1"
+
+    @addmeth()
+    def method2() -> str:
+        order.append("2")
+        return "2"
+
+    @addmeth(trylast=True)
+    def method3() -> str:
+        order.append("3")
+        return "3"
+
+    @addmeth(wrapper=True, tryfirst=True)
+    def method4() -> Generator[None, str, str]:
+        order.append("4pre")
+        result = yield
+        order.append("4post")
+        return result
+
+    @addmeth(wrapper=True)
+    def method5() -> Generator[None, str, str]:
+        order.append("5pre")
+        result = yield
+        order.append("5post")
+        return result
+
+    @addmeth(wrapper=True, trylast=True)
+    def method6() -> Generator[None, str, str]:
+        order.append("6pre")
+        result = yield
+        order.append("6post")
+        return result
+
+    def extra1() -> str:
+        order.append("extra1")
+        return "extra1"
+
+    def extra2() -> str:
+        order.append("extra2")
+        return "extra2"
+
+    result = hc.call_extra([extra1, extra2], {"arg": "test"})
+    assert order == [
+        "4pre",
+        "5pre",
+        "6pre",
+        "1",
+        "extra2",
+        "extra1",
+        "2",
+        "3",
+        "6post",
+        "5post",
+        "4post",
+    ]
+    assert result == [
+        "1",
+        "extra2",
+        "extra1",
+        "2",
+        "3",
+    ]
